/*
 * Copyright 2006 The Xanot team
 */
package org.xanot;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

import org.xanot.annotation.ParentReference;
import org.xanot.annotation.Repeatable;
import org.xanot.annotation.TagInstance;
import org.xanot.annotation.XmlAttribute;
import org.xanot.annotation.XmlCollectionProperty;
import org.xanot.annotation.XmlSingleProperty;
import org.xanot.structure.AttributeSetRule;
import org.xanot.structure.CloseCollectionInstanceRule;
import org.xanot.structure.CloseInstanceRule;
import org.xanot.structure.CreateInstanceRule;
import org.xanot.structure.ParentReferenceRule;
import org.xanot.structure.PropertyCollectionSetRule;
import org.xanot.structure.PropertySetRule;
import org.xanot.structure.XanotRule;

/**
 * <p>
 * Generate an object value and creation rule based on the annotation provided
 * within the object model.
 * </p>
 * <p/>
 * <p>
 * It start on a root object. The top most object that will be instantiated
 * first.
 * </p>
 * <p/>
 * <p>
 * The rule builder then will iterates through all public getter methods and
 * member attributes.
 * </p>
 * 
 * @author Ferdinand Neman (newm4n_at_gmail.com)
 */
public class AnnotationRuleBuilder implements RuleBuilder {
	private List<Class> lock = new ArrayList<Class>();

	private List<XanotRule> rules = new ArrayList<XanotRule>();

	private Class root;

	/**
	 * Creates a new AnnotationRuleBuilder object.
	 * 
	 * @param root
	 *            The root class as the starting point.
	 * @throws XanotException
	 *             Thrown if there's an invalid usage of annotation according to
	 *             this implementation
	 */
	public AnnotationRuleBuilder(Class root) throws XanotException {
		this.root = root;
		buildUpRule(root, null);
	}

	/**
	 * Get the root class
	 * 
	 * @return Root class, never return null
	 */
	public Class getRoot() {
		return root;
	}

	/**
	 * Dump all information about rules this builder contains. All information
	 * is dumped to standard output.
	 */
	public void dumpRulesInfo() {
		for (XanotRule rule : rules) {
			System.out.println(rule.toString());
		}
	}

	/**
	 * Method that buildup rules. This method is called recursively. This method
	 * is working more like a spider. It continues to follow all method return
	 * type annotated with XmlSingleProperty, XmlAttribute or
	 * XmlCollectionProperty as long as the return is not a type of collection
	 * frame work and native/wrappers type.
	 * 
	 * @param klazz
	 *            The source class to build up.
	 * @param parentPath
	 *            Current parrent path.
	 * @throws XanotException
	 *             Thrown if there's an invalid usage of annotation according to
	 *             this implementation
	 */
	@SuppressWarnings("unchecked")
	private void buildUpRule(Class klazz, String parentPath)
			throws XanotException {
		String classXmlPath = "";

		// Check for TagInstance annotation in this class.
		if (klazz.isAnnotationPresent(TagInstance.class)) {
			TagInstance annot = (TagInstance) klazz
					.getAnnotation(TagInstance.class);
			boolean isRoot = false;
			boolean isRepeatable = false;

			if ((parentPath == null) || (parentPath.trim().length() == 0)) {
				classXmlPath = annot.path();
				isRoot = true;
			} else {
				classXmlPath = parentPath + "/" + annot.path();
			}

			if (klazz.isAnnotationPresent(Repeatable.class)) {
				classXmlPath = "../" + annot.path();
				isRepeatable = true;
			} else if (isRoot) {
				classXmlPath = "/" + classXmlPath;
			}

			CreateInstanceRule cir = new CreateInstanceRule(classXmlPath, klazz);
			cir.setRoot(isRoot);
			cir.setRepeatable(isRepeatable);
			rules.add(cir);
		} else {
			if (parentPath == null || parentPath.trim().length() == 0) {
				throw new XanotException(
						klazz.getName()
								+ " is not annotated with @TagInstance, Therefore it cant be a root.");
			}
			classXmlPath = parentPath;
			CreateInstanceRule cir = new CreateInstanceRule(classXmlPath, klazz);
			cir.setRoot(false);
			cir.setRepeatable(false);
			rules.add(cir);
		}

		// Add the annotation for each "get" method.
		for (Method method : klazz.getMethods()) {
			if (method.getName().startsWith("get")
					|| method.getName().startsWith("is")) {
				if (method.isAnnotationPresent(ParentReference.class)) {
					ParentReferenceRule rule = new ParentReferenceRule(
							classXmlPath, getPropertyOfGetter(method.getName()));
					rules.add(rule);
				} else if (method.isAnnotationPresent(XmlAttribute.class)) {
					XmlAttribute annot = method
							.getAnnotation(XmlAttribute.class);
					AttributeSetRule rule = new AttributeSetRule(classXmlPath,
							getPropertyOfGetter(method.getName()), annot.name());
					rule.setDateFormat(annot.dateFormat());
					rules.add(rule);
				} else if (method.isAnnotationPresent(XmlSingleProperty.class)) {
					Class retType = method.getReturnType();

					if (isNativeWrapper(retType)) {
						XmlSingleProperty anot = method
								.getAnnotation(XmlSingleProperty.class);
						PropertySetRule rule = new PropertySetRule(classXmlPath
								+ "/" + anot.name(), getPropertyOfGetter(method
								.getName()));
						rule.setDateFormat(anot.dateFormat());
						rules.add(rule);
					} else if (isCollectionWrapper(retType)) {
						throw new XanotException(
								method.getName()
										+ " is returning collection. Its must not annotated with @XmlSingleProperty use @XmlCollectionProperty instead");
					} else {
						if (retType.isAnnotationPresent(TagInstance.class)) {
							TagInstance annot = (TagInstance) retType
									.getAnnotation(TagInstance.class);
							CloseInstanceRule rule = new CloseInstanceRule(
									classXmlPath + "/" + annot.path(),
									getPropertyOfGetter(method.getName()));
							rules.add(rule);

							if (!lock.contains(retType)) {
								lock.add(retType);
								buildUpRule(retType, classXmlPath);
								lock.remove(retType);
							}
						} else {
							if (lock.contains(retType)) {
								throw new XanotException(
										retType.getName()
												+ " was referenced and not annotated with  @TagInstance and referenced in circular.");
							}
							XmlSingleProperty anot = method
									.getAnnotation(XmlSingleProperty.class);
							CloseInstanceRule rule = new CloseInstanceRule(
									classXmlPath + "/" + anot.name(),
									getPropertyOfGetter(method.getName()));
							rule.setDateFormat(anot.dateFormat());
							rules.add(rule);

							lock.add(retType);
							buildUpRule(retType, classXmlPath + "/"
									+ anot.name());
							lock.remove(retType);
						}
					}
				} else if (method
						.isAnnotationPresent(XmlCollectionProperty.class)) {
					Class retType = method.getReturnType();
					XmlCollectionProperty annot = method
							.getAnnotation(XmlCollectionProperty.class);

					if (isCollectionWrapper(retType)) {
						if (isNativeWrapper(annot.element())) {
							PropertyCollectionSetRule rule = new PropertyCollectionSetRule(
									classXmlPath + "/" + annot.xmltag(), annot
											.xmltag(), annot.methodName());
							rule.setDateFormat(annot.dateFormat());
							rules.add(rule);
						} else {
							if (annot.element().equals(klazz)) {
								CloseCollectionInstanceRule rule = new CloseCollectionInstanceRule(
										classXmlPath, annot.methodName());
								rule.setDateFormat(annot.dateFormat());
								rules.add(rule);
							} else if (annot.element().isAnnotationPresent(
									TagInstance.class)) {
								TagInstance subanot = (TagInstance) annot
										.element().getAnnotation(
												TagInstance.class);
								CloseCollectionInstanceRule rule = new CloseCollectionInstanceRule(
										classXmlPath + "/" + subanot.path(),
										annot.methodName());
								rule.setDateFormat(annot.dateFormat());
								rules.add(rule);

								if (!isCreateInstanceRuleExist(annot.element())) {
									buildUpRule(annot.element(), classXmlPath);
								}
							} else {
								if (lock.contains(retType)) {
									throw new XanotException(
											retType.getName()
													+ " was referenced and not annotated with  @TagInstance and referenced in circular.");
								}
								CloseCollectionInstanceRule rule = new CloseCollectionInstanceRule(
										classXmlPath + "/" + annot.xmltag(),
										annot.methodName());
								rule.setDateFormat(annot.dateFormat());
								rules.add(rule);

								lock.add(retType);
								buildUpRule(annot.element(), classXmlPath + "/"
										+ annot.xmltag());
								lock.remove(retType);
							}
						}
					} else {
						throw new XanotException(
								method.getName()
										+ " is returning non collection. use @XmlSingleProperty instead");
					}
				}
			}
		}
	}

	/**
	 * Check wether a class is a root class of the object model.
	 * 
	 * @param klazz
	 *            Class to check
	 * @return True if the class is a root class. False if otherwise.
	 */
	public boolean isRoot(Class klazz) {
		for (XanotRule rule : rules) {
			if (rule instanceof CreateInstanceRule) {
				CreateInstanceRule r = (CreateInstanceRule) rule;

				return r.isRoot();
			}
		}

		return false;
	}

	/**
	 * Check wether a class is registered as a create instance class.
	 * 
	 * @param klazz
	 *            Class to check
	 * @return True if the class is used to create an instance, false if
	 *         otherwise.
	 */
	private boolean isCreateInstanceRuleExist(Class klazz) {
		for (XanotRule rule : rules) {
			if (rule instanceof CreateInstanceRule) {
				CreateInstanceRule r = (CreateInstanceRule) rule;

				if (r.getBaseClass().equals(klazz)) {
					return true;
				}
			}
		}

		return false;
	}

	/**
	 * To check wether a class extends a collection typed class.
	 * 
	 * @param klazz
	 *            The class to check
	 * @return True if the class is implementing List or Set interface.
	 */
	private boolean isCollectionWrapper(Class klazz) {
		if (Set.class.isAssignableFrom(klazz)
				|| List.class.isAssignableFrom(klazz)) {
			return true;
		}

		return false;
	}

	/**
	 * To check wether a class is extending or a wrapper class of native tipes.
	 * 
	 * @param klazz
	 *            The class to check.
	 * @return True if the class is instance of Byte, Short, Integer, Long,
	 *         Double, Float,Boolean or even an int,
	 *         byte,short,long,float,double,char or boolean.
	 */
	public static boolean isNativeWrapper(Class klazz) {
		if (Byte.class.isAssignableFrom(klazz)
				|| Short.class.isAssignableFrom(klazz)
				|| Integer.class.isAssignableFrom(klazz)
				|| Long.class.isAssignableFrom(klazz)
				|| Float.class.isAssignableFrom(klazz)
				|| Double.class.isAssignableFrom(klazz)
				|| String.class.isAssignableFrom(klazz)
				|| Date.class.isAssignableFrom(klazz)
				|| Character.class.isAssignableFrom(klazz)
				|| Boolean.class.isAssignableFrom(klazz)
				|| klazz.getName().equals("byte")
				|| klazz.getName().equals("short")
				|| klazz.getName().equals("int")
				|| klazz.getName().equals("long")
				|| klazz.getName().equals("float")
				|| klazz.getName().equals("double")
				|| klazz.getName().equals("char")
				|| klazz.getName().equals("boolean")) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Get the property name of a getter method. For example
	 * 
	 * <pre><code>
	 *          foo  &lt;- getFoo
	 *   bar  &lt;- getBar
	 *   or
	 *   foo  &lt;- isFoo
	 *   bar  &lt;- isBar
	 * </code></pre>
	 * 
	 * @param getterName
	 *            Getter method name
	 * @return Property Name
	 * @throws XanotException
	 *             Thrown if the profided getter method is not a getter method.
	 *             as if its not started with &quot;get&quot; or &quot;is&quot;
	 */
	private String getPropertyOfGetter(String getterName) throws XanotException {
		String propertyNameCapital = "";

		if (getterName.startsWith("get")) {
			propertyNameCapital = getterName.substring(3);
		} else if (getterName.startsWith("is")) {
			propertyNameCapital = getterName.substring(2);
		} else {
			throw new XanotException("Not a getter method. " + getterName);
		}

		return Character.toLowerCase(propertyNameCapital.charAt(0))
				+ propertyNameCapital.substring(1);
	}

	/**
	 * Get attribute setter with specified path.
	 * 
	 * @param path
	 *            Rule Path
	 * @return AttributeSetRule instance or null no Attribute Set rule with that
	 *         path.
	 */
	public AttributeSetRule getAttributeSetRule(String path) {
		for (XanotRule anot : rules) {
			if (anot.getPath().equals(path)) {
				if (anot instanceof AttributeSetRule) {
					return (AttributeSetRule) anot;
				}
			}
		}

		return null;
	}

	/**
	 * Simple helper method to check wether a specific object is contained
	 * inside an array.
	 * 
	 * @param arr
	 *            Array containing objects
	 * @param o
	 *            Object to check
	 * @return True if the array is containing the specified object.
	 */
	private boolean isArrayContains(Object[] arr, Object o) {
		for (Object obj : arr) {
			if (obj.equals(o)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Get array of <code>AttributeSetRule</code> that have a path similar to
	 * an array of paths.
	 * 
	 * @param path
	 *            Array of paths.
	 * @return Array of <code>AttributeSetRule</code> that have a path similar
	 *         to the array of path.
	 */
	public AttributeSetRule[] getAttributeSetRule(String[] path) {
		ArrayList<AttributeSetRule> al = new ArrayList<AttributeSetRule>();

		for (XanotRule anot : rules) {
			if (isArrayContains(path, anot.getPath())) {
				if (anot instanceof AttributeSetRule) {
					al.add((AttributeSetRule) anot);
				}
			}
		}

		return al.toArray(new AttributeSetRule[al.size()]);
	}

	/**
	 * Get closer instance with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return CloseInstanceRule that it's path is equals to the specified path
	 */
	public CloseInstanceRule getCloseInstanceRule(String path) {
		for (XanotRule anot : rules) {
			if (anot.getPath().equals(path)) {
				if (anot instanceof AttributeSetRule) {
					return (CloseInstanceRule) anot;
				}
			}
		}

		return null;
	}

	/**
	 * Get array of <code>CloseInstanceRule</code> that have a path similar to
	 * an array of paths.
	 * 
	 * @param path
	 *            Array of paths.
	 * @return Array of <code>CloseInstanceRule</code> that have a path
	 *         similar to the array of path.
	 */
	public CloseInstanceRule[] getCloseInstanceRule(String[] path) {
		ArrayList<CloseInstanceRule> al = new ArrayList<CloseInstanceRule>();

		for (XanotRule anot : rules) {
			if (isArrayContains(path, anot.getPath())) {
				if (anot instanceof CloseInstanceRule) {
					al.add((CloseInstanceRule) anot);
				}
			}
		}

		return al.toArray(new CloseInstanceRule[al.size()]);
	}

	/**
	 * Get <code>CreateInstanceRule</code> with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return <code>CreateInstanceRule</code> that it's path is equals to the
	 *         specified path
	 */
	public CreateInstanceRule getCreateInstanceRule(String path) {
		for (XanotRule anot : rules) {
			if (anot.getPath().equals(path)) {
				if (anot instanceof CreateInstanceRule) {
					return (CreateInstanceRule) anot;
				}
			}
		}

		return null;
	}

	/**
	 * Get array of <code>CreateInstanceRule</code> that have a path similar
	 * to an array of paths.
	 * 
	 * @param path
	 *            Array of paths.
	 * @return Array of <code>CreateInstanceRule</code> that have a path
	 *         similar to the array of path.
	 */
	public CreateInstanceRule[] getCreateInstanceRule(String[] path) {
		ArrayList<CreateInstanceRule> al = new ArrayList<CreateInstanceRule>();

		for (XanotRule anot : rules) {
			if (isArrayContains(path, anot.getPath())) {
				if (anot instanceof CreateInstanceRule) {
					al.add((CreateInstanceRule) anot);
				}
			}
		}

		return al.toArray(new CreateInstanceRule[al.size()]);
	}

	/**
	 * Get <code>PropertySetRule</code> with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return <code>PropertySetRule</code> that it's path is equals to the
	 *         specified path
	 */
	public PropertySetRule getPropertySetRule(String path) {
		for (XanotRule anot : rules) {
			if (anot.getPath().equals(path)) {
				if (anot instanceof PropertySetRule) {
					return (PropertySetRule) anot;
				}
			}
		}

		return null;
	}

	/**
	 * Get array of <code>PropertySetRule</code> that have a path similar to
	 * an array of paths.
	 * 
	 * @param path
	 *            Array of paths.
	 * @return Array of <code>PropertySetRule</code> that have a path similar
	 *         to the array of path.
	 */
	public PropertySetRule[] getPropertySetRule(String[] path) {
		ArrayList<PropertySetRule> al = new ArrayList<PropertySetRule>();

		for (XanotRule anot : rules) {
			if (isArrayContains(path, anot.getPath())) {
				if (anot instanceof PropertySetRule) {
					al.add((PropertySetRule) anot);
				}
			}
		}

		return al.toArray(new PropertySetRule[al.size()]);
	}

	/**
	 * Get <code>CloseCollectionInstanceRule</code> with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return <code>CloseCollectionInstanceRule</code> that it's path is
	 *         equals to the specified path
	 */
	public CloseCollectionInstanceRule getCloseCollectionInstanceRule(
			String path) {
		for (XanotRule anot : rules) {
			if (anot.getPath().equals(path)) {
				if (anot instanceof CloseCollectionInstanceRule) {
					return (CloseCollectionInstanceRule) anot;
				}
			}
		}

		return null;
	}

	/**
	 * Get array of <code>CloseCollectionInstanceRule</code> that have a path
	 * similar to an array of paths.
	 * 
	 * @param path
	 *            Array of paths.
	 * @return Array of <code>CloseCollectionInstanceRule</code> that have a
	 *         path similar to the array of path.
	 */
	public CloseCollectionInstanceRule[] getCloseCollectionInstanceRule(
			String[] path) {
		ArrayList<CloseCollectionInstanceRule> al = new ArrayList<CloseCollectionInstanceRule>();

		for (XanotRule anot : rules) {
			if (isArrayContains(path, anot.getPath())) {
				if (anot instanceof CloseCollectionInstanceRule) {
					al.add((CloseCollectionInstanceRule) anot);
				}
			}
		}

		return al.toArray(new CloseCollectionInstanceRule[al.size()]);
	}

	/**
	 * Get <code>PropertyCollectionSetRule</code> with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return <code>PropertyCollectionSetRule</code> that it's path is equals
	 *         to the specified path
	 */
	public PropertyCollectionSetRule getPropertyCollectionSetRule(String path) {
		for (XanotRule anot : rules) {
			if (anot.getPath().equals(path)) {
				if (anot instanceof PropertyCollectionSetRule) {
					return (PropertyCollectionSetRule) anot;
				}
			}
		}

		return null;
	}

	/**
	 * Get array of <code>PropertyCollectionSetRule</code> that have a path
	 * similar to an array of paths.
	 * 
	 * @param path
	 *            Array of paths.
	 * @return Array of <code>PropertyCollectionSetRule</code> that have a
	 *         path similar to the array of path.
	 */
	public PropertyCollectionSetRule[] getPropertyCollectionSetRule(
			String[] path) {
		ArrayList<PropertyCollectionSetRule> al = new ArrayList<PropertyCollectionSetRule>();

		for (XanotRule anot : rules) {
			if (isArrayContains(path, anot.getPath())) {
				if (anot instanceof PropertyCollectionSetRule) {
					al.add((PropertyCollectionSetRule) anot);
				}
			}
		}

		return al.toArray(new PropertyCollectionSetRule[al.size()]);
	}

	/**
	 * Get <code>ParentReferenceRule</code> with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return <code>ParentReferenceRule</code> that it's path is equals to
	 *         the specified path
	 */
	public ParentReferenceRule getParentReferenceRule(String path) {
		for (XanotRule anot : rules) {
			if (anot.getPath().equals(path)) {
				if (anot instanceof ParentReferenceRule) {
					return (ParentReferenceRule) anot;
				}
			}
		}
		return null;
	}

	/**
	 * Get array of <code>ParentReferenceRule</code> that have a path similar
	 * to an array of paths.
	 * 
	 * @param path
	 *            Array of paths.
	 * @return Array of <code>ParentReferenceRule</code> that have a path
	 *         similar to the array of path.
	 */
	public ParentReferenceRule[] getParentReferenceRule(String[] paths) {
		ArrayList<ParentReferenceRule> al = new ArrayList<ParentReferenceRule>();

		for (XanotRule anot : rules) {
			if (isArrayContains(paths, anot.getPath())) {
				if (anot instanceof ParentReferenceRule) {
					al.add((ParentReferenceRule) anot);
				}
			}
		}

		return al.toArray(new ParentReferenceRule[al.size()]);
	}

	/**
	 * Get List of <code>XanotRule</code> with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return List of <code>XanotRule</code> that it's path is equals to the
	 *         specified path
	 */
	public List<XanotRule> getRules(String path) {
		ArrayList<XanotRule> al = new ArrayList<XanotRule>();

		for (XanotRule anot : rules) {
			if (anot.getPath().equals(path)) {
				al.add(anot);
			}
		}

		return al;
	}

	/**
	 * Get List of <code>XanotRule</code> that have a path similar to an array
	 * of paths.
	 * 
	 * @param paths
	 *            Array of paths.
	 * @return List of <code>XanotRule</code> that have a path similar to the
	 *         array of path.
	 */
	public List<XanotRule> getRules(String[] paths) {
		ArrayList<XanotRule> al = new ArrayList<XanotRule>();

		for (XanotRule anot : rules) {
			if (isArrayContains(paths, anot.getPath())) {
				al.add(anot);
			}
		}

		return al;
	}
}
