/*
 * Copyright 2006 The Xanot team
 */
package org.xanot;

import java.util.List;

import org.xanot.structure.AttributeSetRule;
import org.xanot.structure.CloseCollectionInstanceRule;
import org.xanot.structure.CloseInstanceRule;
import org.xanot.structure.CreateInstanceRule;
import org.xanot.structure.ParentReferenceRule;
import org.xanot.structure.PropertyCollectionSetRule;
import org.xanot.structure.PropertySetRule;
import org.xanot.structure.XanotRule;

/**
 * <p>
 * Generate an object value and creation rule
 * </p>
 * 
 * @author Ferdinand Neman (newm4n_at_gmail.com)
 */
public interface RuleBuilder {
	/**
	 * Get the root class
	 * 
	 * @return Root class, never return null
	 */
	public Class getRoot();

	/**
	 * Get List of <code>XanotRule</code> with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return List of <code>XanotRule</code> that it's path is equals to the
	 *         specified path
	 */
	public List<XanotRule> getRules(String path);

	/**
	 * Get List of <code>XanotRule</code> that have a path similar to an array
	 * of paths.
	 * 
	 * @param paths
	 *            Array of paths.
	 * @return List of <code>XanotRule</code> that have a path similar to the
	 *         array of path.
	 */
	public List<XanotRule> getRules(String[] paths);

	/**
	 * Get <code>CreateInstanceRule</code> with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return <code>CreateInstanceRule</code> that it's path is equals to the
	 *         specified path
	 */
	public CreateInstanceRule getCreateInstanceRule(String path);

	/**
	 * Get array of <code>CreateInstanceRule</code> that have a path similar
	 * to an array of paths.
	 * 
	 * @param paths
	 *            Array of paths.
	 * @return Array of <code>CreateInstanceRule</code> that have a path
	 *         similar to the array of path.
	 */
	public CreateInstanceRule[] getCreateInstanceRule(String[] paths);

	/**
	 * Get closer instance with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return CloseInstanceRule that it's path is equals to the specified path
	 */
	public CloseInstanceRule getCloseInstanceRule(String path);

	/**
	 * Get array of <code>CloseInstanceRule</code> that have a path similar to
	 * an array of paths.
	 * 
	 * @param path
	 *            Array of paths.
	 * @return Array of <code>CloseInstanceRule</code> that have a path
	 *         similar to the array of path.
	 */
	public CloseInstanceRule[] getCloseInstanceRule(String[] path);

	/**
	 * Get <code>PropertySetRule</code> with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return <code>PropertySetRule</code> that it's path is equals to the
	 *         specified path
	 */
	public PropertySetRule getPropertySetRule(String path);

	/**
	 * Get array of <code>PropertySetRule</code> that have a path similar to
	 * an array of paths.
	 * 
	 * @param path
	 *            Array of paths.
	 * @return Array of <code>PropertySetRule</code> that have a path similar
	 *         to the array of path.
	 */
	public PropertySetRule[] getPropertySetRule(String[] path);

	/**
	 * Get attribute setter with specified path.
	 * 
	 * @param path
	 *            Rule Path
	 * @return AttributeSetRule instance or null no Attribute Set rule with that
	 *         path.
	 */
	public AttributeSetRule getAttributeSetRule(String path);

	/**
	 * Get array of <code>AttributeSetRule</code> that have a path similar to
	 * an array of paths.
	 * 
	 * @param path
	 *            Array of paths.
	 * @return Array of <code>AttributeSetRule</code> that have a path similar
	 *         to the array of path.
	 */
	public AttributeSetRule[] getAttributeSetRule(String[] path);

	/**
	 * Get <code>CloseCollectionInstanceRule</code> with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return <code>CloseCollectionInstanceRule</code> that it's path is
	 *         equals to the specified path
	 */
	public CloseCollectionInstanceRule getCloseCollectionInstanceRule(
			String path);

	/**
	 * Get array of <code>CloseCollectionInstanceRule</code> that have a path
	 * similar to an array of paths.
	 * 
	 * @param path
	 *            Array of paths.
	 * @return Array of <code>CloseCollectionInstanceRule</code> that have a
	 *         path similar to the array of path.
	 */
	public CloseCollectionInstanceRule[] getCloseCollectionInstanceRule(
			String[] path);

	/**
	 * Get <code>PropertyCollectionSetRule</code> with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return <code>PropertyCollectionSetRule</code> that it's path is equals
	 *         to the specified path
	 */
	public PropertyCollectionSetRule getPropertyCollectionSetRule(String path);

	/**
	 * Get array of <code>PropertyCollectionSetRule</code> that have a path
	 * similar to an array of paths.
	 * 
	 * @param path
	 *            Array of paths.
	 * @return Array of <code>PropertyCollectionSetRule</code> that have a
	 *         path similar to the array of path.
	 */
	public PropertyCollectionSetRule[] getPropertyCollectionSetRule(
			String[] path);

	/**
	 * Get <code>CreateInstanceRule</code> with specified path.
	 * 
	 * @param path
	 *            The path
	 * @return <code>CreateInstanceRule</code> that it's path is equals to the
	 *         specified path
	 */
	public ParentReferenceRule getParentReferenceRule(String path);

	/**
	 * Get array of <code>CreateInstanceRule</code> that have a path similar
	 * to an array of paths.
	 * 
	 * @param paths
	 *            Array of paths.
	 * @return Array of <code>CreateInstanceRule</code> that have a path
	 *         similar to the array of path.
	 */
	public ParentReferenceRule[] getParentReferenceRule(String[] paths);
}
