/*
 * Copyright 2006 The Xanot team
 */
package org.xanot;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.ArrayList;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

/**
 * This is the class that, in user point of view, will do all the job.
 * 
 * @author Ferdinand Neman (newm4n_at_gmail.com)
 */
public class Xanot {
	private Class root;

	private RuleBuilder ruleBuilder = null;

	private Collection<SAXParseException> warnings = new ArrayList<SAXParseException>();

	private Collection<SAXParseException> errors = new ArrayList<SAXParseException>();

	private Collection<SAXParseException> fatals = new ArrayList<SAXParseException>();

	private boolean stopOnWarning = true;

	private boolean stopOnError = true;

	private boolean stopOnFatal = true;

	/**
	 * Creates a new Xanot object.
	 * 
	 * @throws XanotException
	 *             DOCUMENT ME!
	 */
	public Xanot() throws XanotException {
	}

	/**
	 * Get the root class
	 * 
	 * @return Returns the root.
	 */
	public Class getRoot() {
		return root;
	}

	/**
	 * Set the root object. This class will be used as the starting point for
	 * xanot to generate mapping rule.
	 * 
	 * @param root
	 *            The root to set.
	 */
	public void setRoot(Class root) throws XanotException {
		this.root = root;
		buildUp(root);
	}

	/**
	 * Internal method that start the rule building process.
	 * 
	 * @param root
	 *            Starting point class.
	 * @throws XanotException
	 *             Thrown if anything goes wrong.
	 */
	private void buildUp(Class root) throws XanotException {
		if (ruleBuilder == null) {
			ruleBuilder = new AnnotationRuleBuilder(root);
		}
	}

	/**
	 * Get this class name. Equals to <code>Xanot.class.getName()</code>
	 * 
	 * @return DOCUMENT ME!
	 */
	public String toString() {
		return Xanot.class.getName();
	}

	/**
	 * The method to call when starting to parse an xml object
	 * 
	 * @param inputStream
	 *            Input stream as source of the xml data.
	 * @return Return the mapped instance of the object
	 * @throws SAXException
	 *             Thrown if there's an error when parsing.
	 * @throws IOException
	 *             Thrown if there's some problem when accesing the stream.
	 * @throws ParserConfigurationException
	 *             Thrown if the sax parser is not properly configured.
	 */
	public Object parse(InputStream inputStream)
			throws ParserConfigurationException, SAXException, IOException {

		XanotSAXHandler handler = new XanotSAXHandler(ruleBuilder);
		handler.setStopOnError(isStopOnError());
		handler.setStopOnFatal(isStopOnFatal());
		handler.setStopOnWarning(isStopOnWarning());

		SAXParserFactory factory = SAXParserFactory.newInstance();

		SAXParser saxParser = factory.newSAXParser();
		saxParser.parse(inputStream, handler);

		setFatals(handler.getFatals());
		setErrors(handler.getErrors());
		setWarnings(handler.getWarnings());

		return handler.getRootObject();
	}

	/**
	 * Start to parse an xml from a source file.
	 * 
	 * @param inputFile
	 *            Input file
	 * @return Mapped instance as the result.
	 * @throws SAXException
	 *             Thrown if there's an error when parsing.
	 * @throws IOException
	 *             Thrown if there's some problem when accessing file.
	 * @throws ParserConfigurationException
	 *             Thrown if the sax parser is not properly configured.
	 */
	public Object parse(File inputFile) throws SAXException, IOException,
			ParserConfigurationException {

		XanotSAXHandler handler = new XanotSAXHandler(ruleBuilder);
		handler.setStopOnError(isStopOnError());
		handler.setStopOnFatal(isStopOnFatal());
		handler.setStopOnWarning(isStopOnWarning());

		SAXParserFactory factory = SAXParserFactory.newInstance();

		SAXParser saxParser = factory.newSAXParser();
		saxParser.parse(inputFile, handler);

		return handler.getRootObject();
	}

	/**
	 * Start to parse an xml from a source file on a specified file path.
	 * 
	 * @param inputFilePath
	 *            Path to the xml file
	 * @return Mapped instance as the result
	 * @throws SAXException
	 *             Thrown if there's an error when parsing.
	 * @throws IOException
	 *             Thrown if there's some problem when accessing file.
	 * @throws ParserConfigurationException
	 *             Thrown if the sax parser is not properly configured.
	 */
	public Object parse(String inputFilePath) throws SAXException, IOException,
			ParserConfigurationException {
		File inputFile = new File(inputFilePath);

		if (inputFile.exists()) {
			return parse(inputFile);
		} else {
			System.out.println("File " + inputFilePath + " not exist");

			return null;
		}
	}

	/**
	 * Get collection of warning exceptions
	 * 
	 * @return Collection of warning exceptions.
	 */
	public Collection<SAXParseException> getWarnings() {
		return warnings;
	}

	/**
	 * Set collection of warning exceptions.
	 * 
	 * @param warnings
	 *            Collection of warning exceptions.
	 */
	public void setWarnings(Collection<SAXParseException> warnings) {
		this.warnings = warnings;
	}

	/**
	 * Get collection of error exceptions
	 * 
	 * @return Collection of error exceptions.
	 */
	public Collection<SAXParseException> getErrors() {
		return errors;
	}

	/**
	 * Set collection of error exceptions
	 * 
	 * @param errors
	 *            Collection of error exceptions.
	 */
	public void setErrors(Collection<SAXParseException> errors) {
		this.errors = errors;
	}

	/**
	 * Get collection of fatal error exceptions.
	 * 
	 * @return Collection of fatal error exception.
	 */
	public Collection<SAXParseException> getFatals() {
		return fatals;
	}

	/**
	 * Set collection of fatal errors exceptions.
	 * 
	 * @param fatals
	 *            Collection of fatal error exception.
	 */
	public void setFatals(Collection<SAXParseException> fatals) {
		this.fatals = fatals;
	}

	/**
	 * Check wether xanot will stop parsing xml when encounters a warning.
	 * 
	 * @return True if xanot will stop parsing on warning. False if otherwise.
	 */
	public boolean isStopOnWarning() {
		return stopOnWarning;
	}

	/**
	 * Set xanot to stop or continue parasing when encounters a warning.
	 * 
	 * @param stopOnWarning
	 *            True to stop xanot when encounter a warning. False if
	 *            otherwise.
	 */
	public void setStopOnWarning(boolean stopOnWarning) {
		this.stopOnWarning = stopOnWarning;
	}

	/**
	 * Check wether xanot will stop parsing xml when encounters an error.
	 * 
	 * @return True if xanot will stop parsing on error. False if otherwise.
	 */
	public boolean isStopOnError() {
		return stopOnError;
	}

	/**
	 * Set xanot to stop or continue parasing when encounters an error.
	 * 
	 * @param stopOnError
	 *            True to stop xanot when encounter an error. False if
	 *            otherwise.
	 */
	public void setStopOnError(boolean stopOnError) {
		this.stopOnError = stopOnError;
	}

	/**
	 * Check wether xanot will stop parsing xml when encounters a fatal error.
	 * 
	 * @return True if xanot will stop parsing on fatal error. False if
	 *         otherwise.
	 */
	public boolean isStopOnFatal() {
		return stopOnFatal;
	}

	/**
	 * Set xanot to stop or continue parasing when encounters a fatal error.
	 * 
	 * @param stopOnFatal
	 *            True to stop xanot when encounter a fatal error. False if
	 *            otherwise.
	 */
	public void setStopOnFatal(boolean stopOnFatal) {
		this.stopOnFatal = stopOnFatal;
	}
}
