package org.xanot.dtd;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Implementation of the DTD model.
 * 
 * @author Ferdinand Neman (newm4n_at_gmail.com)
 * 
 */
public class DefaultDtdModel implements DtdModel {

	/**
	 * Main pattern of DTD element patterns. This pattern include elements of
	 * attlist and element.
	 */
	public static String DTD_PATTERN = "<!(ATTLIST|ELEMENT)\\s+[\\S\\s&&[^>]]+>";

	/**
	 * The pattern object compiled from <code>DTD_PATTERN</code>
	 */
	public static Pattern pattern = Pattern.compile(DTD_PATTERN);

	private ArrayList<DtdAttList> dtdAttLists = new ArrayList<DtdAttList>();

	private ArrayList<DtdElement> dtdElements = new ArrayList<DtdElement>();

	/**
	 * Create new instance of Dtd model.
	 * 
	 * @param path
	 *            Path to the dtd file.
	 * @throws IOException
	 *             Thrown if some how the dtd file canot be readed.
	 */
	public DefaultDtdModel(String path) throws IOException {
		InputStream is;
		is = DefaultDtdModel.class.getResourceAsStream(path);
		if (is == null) {
			File file = new File(path);
			if (file.exists()) {
				is = new FileInputStream(file);
			}
		}
		if (is != null) {
			parseIn(is);
		} else {
			throw new FileNotFoundException(path);
		}
	}

	/**
	 * Create new instance of Dtd Model.
	 * 
	 * @param file
	 *            DTD file object.
	 * @throws IOException
	 *             Thrown if somehow the dtd file canot be readed.
	 */
	public DefaultDtdModel(File file) throws IOException {
		FileInputStream fis = new FileInputStream(file);
		parseIn(fis);
		fis.close();
	}

	/**
	 * Create new instance of Dtd Model.
	 * 
	 * @param inputStream
	 *            Input stream where streamed in data is a DTD data.
	 * @throws IOException
	 *             Thrown if somehow problem raised because of streaming-in.
	 */
	public DefaultDtdModel(InputStream inputStream) throws IOException {
		parseIn(inputStream);
	}

	private void parseIn(InputStream inputStream) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		byte[] buffer = new byte[200];
		int size = 0;
		while (size >= 0) {
			size = inputStream.read(buffer);
			if (size > 0) {
				baos.write(buffer, 0, size);
			}
		}

		String data = new String(baos.toByteArray());
		Matcher match = pattern.matcher(data);
		while (match.find()) {
			String tag = match.group();
			if (tag.startsWith("<!ATTLIST")) {
				dtdAttLists.add(new DtdAttList(tag));
			} else {
				dtdElements.add(new DtdElement(tag));
			}
		}
	}

	/**
	 * @see org.xanot.dtd.DtdModel#getDtdAttLists()
	 */
	public ArrayList<DtdAttList> getDtdAttLists() {
		return dtdAttLists;
	}

	/**
	 * Set DtdAttLists Collections.
	 * 
	 * @param dtdAttLists
	 *            New DtdAttList object array list.
	 */
	public void setDtdAttLists(ArrayList<DtdAttList> dtdAttLists) {
		this.dtdAttLists = dtdAttLists;
	}

	/**
	 * @see org.xanot.dtd.DtdModel#getDtdElements()
	 */
	public ArrayList<DtdElement> getDtdElements() {
		return dtdElements;
	}

	/**
	 * Set DtdElement Collection.
	 * 
	 * @param dtdElements
	 *            New DtdElement object array list.
	 */
	public void setDtdElements(ArrayList<DtdElement> dtdElements) {
		this.dtdElements = dtdElements;
	}

	/**
	 * Add new DtdAttList object into the DtdAttLists array list.
	 * 
	 * @param dtdAttList
	 *            Object to add.
	 */
	public void addDtdAttList(DtdAttList dtdAttList) {
		dtdAttLists.add(dtdAttList);
	}

	/**
	 * Add new DtdElement object into the DtdElements array list.
	 * 
	 * @param dtdElement
	 *            Object to add.
	 */
	public void addDtdElement(DtdElement dtdElement) {
		dtdElements.add(dtdElement);
	}

	/**
	 * Remove an instance from DtdAttLists array list.
	 * 
	 * @param dtdAttList
	 *            Instance to remove.
	 */
	public void removeDtdAttList(DtdAttList dtdAttList) {
		dtdAttLists.remove(dtdAttList);
	}

	/**
	 * Remove an instance from DtdElements array list.
	 * 
	 * @param dtdElement
	 *            Instance to remove.
	 */
	public void removeDtdElement(DtdElement dtdElement) {
		dtdElements.remove(dtdElement);
	}

	/**
	 * @see org.xanot.dtd.DtdModel#dtdAttListCount()
	 */
	public int dtdAttListCount() {
		return dtdAttLists.size();
	}

	/**
	 * @see org.xanot.dtd.DtdModel#dtdElementCount()
	 */
	public int dtdElementCount() {
		return dtdElements.size();
	}

	/**
	 * @see org.xanot.dtd.DtdModel#getDtdAttListAt(int)
	 */
	public DtdAttList getDtdAttListAt(int idx) {
		return dtdAttLists.get(idx);
	}

	/**
	 * @see org.xanot.dtd.DtdModel#getDtdElementAt(int)
	 */
	public DtdElement getDtdElementAt(int idx) {
		return dtdElements.get(idx);
	}

	/**
	 * @see org.xanot.dtd.DtdModel#getDtdElement(java.lang.String)
	 */
	public DtdElement getDtdElement(String name) {
		for (DtdElement element : dtdElements) {
			if (element.getElementName().equals(name)) {
				return element;
			}
		}
		return null;
	}

}
