package org.xanot.dtd;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * DtdAttList model file. This model represent ATTLIST data inside DTD.
 * 
 * @author Ferdinand Neman (newm4n_at_gmail.com)
 * 
 */
public class DtdAttList {
	/**
	 * Attribute Name regex pattern : (\w([\w\-]*\w)*)
	 */
	public static String ATTRIBUTE_NAME = "(\\w([\\w\\-]*\\w)*)";

	/**
	 * An attribute list name pattern :
	 * ([\S&&[^\|\(\)]]{1,2}|[\S&&[^\|\(\)]][\s\S&&[^\|\(\)]]+[\S&&[^\|\(\)]])
	 */
	public static String NAME_WITH_SPACE = "([\\S&&[^\\|\\(\\)]]{1,2}|[\\S&&[^\\|\\(\\)]][\\s\\S&&[^\\|\\(\\)]]+[\\S&&[^\\|\\(\\)]])";

	/**
	 * AttList types enum regex pattern :
	 * \(\s*"+NAME_WITH_SPACE+"(\s*\|\s*"+NAME_WITH_SPACE+")*\s*\)
	 */
	public static String TYPE_ENUM = "\\(\\s*" + NAME_WITH_SPACE
			+ "(\\s*\\|\\s*" + NAME_WITH_SPACE + ")*\\s*\\)";

	/**
	 * CDATA pattern : CDATA
	 */
	public static String CDATA = "CDATA";

	/**
	 * ID pattern : ID
	 */
	public static String ID = "ID";

	/**
	 * IDREF pattern : IDREF
	 */
	public static String IDREF = "IDREF";

	/**
	 * IDREFS pattern : IDREFS
	 */
	public static String IDREFS = "IDREFS";

	/**
	 * NMTOKEN pattern : NMTOKEN
	 */
	public static String NMTOKEN = "NMTOKEN";

	/**
	 * NMTOKENS pattern : NMTOKENS
	 */
	public static String NMTOKENS = "NMTOKENS";

	/**
	 * ENTITY pattern : ENTITY
	 */
	public static String ENTITY = "ENTITY";

	/**
	 * ENTITIES pattern : ENTITIES
	 */
	public static String ENTITIES = "ENTITIES";

	/**
	 * NOTATION pattern : NOTATION
	 */
	public static String NOTATION = "NOTATION";

	/**
	 * Attribute types pattern :
	 * (CDATA|ID|IDREF|IDREFS|NMTOKEN|NMTOKENS|ENTITY|ENTITIES|NOTATION|"+TYPE_ENUM+")
	 */
	public static String ATTRIBUTE_TYPE = "(" + CDATA + "|" + ID + "|" + IDREF
			+ "|" + IDREFS + "|" + NMTOKEN + "|" + NMTOKENS + "|" + ENTITY
			+ "|" + ENTITIES + "|" + NOTATION + "|" + TYPE_ENUM + ")";

	/**
	 * Pattern for AttList which is an "open value" : \"[\s\S]+\"
	 */
	public static String OPEN_VALUE = "\"[\\s\\S]+\"";

	/**
	 * Pattern for AttList which is fixed : #FIXED\s \"[\s\S]+\"
	 */
	public static String FIXED_VALUE = "#FIXED\\s+" + OPEN_VALUE;

	/**
	 * Pattern for AttList which is required : #REQUIRED
	 */
	public static String REQUIRED_VALUE = "#REQUIRED";

	/**
	 * Pattern for AttList which is implied : #IMPLIED
	 */
	public static String IMPLIED_VALUE = "#IMPLIED";

	/**
	 * Pattern for AttList for default value :
	 * (\"[\s\S]+\"|#REQUIRED|#IMPLIED|#FIXED\s \"[\s\S]+\")
	 */
	public static String DEVAULT_VALUE = "(" + OPEN_VALUE + "|"
			+ REQUIRED_VALUE + "|" + IMPLIED_VALUE + "|" + FIXED_VALUE + ")";

	/**
	 * Pattern for attlist : ATTLIST
	 */
	public static String ATTLIST = "ATTLIST";

	/**
	 * Pattern for complete attlist pattern
	 */
	public static String ATTLIST_PATTERN = "<!" + ATTLIST + "\\s+"
			+ DtdElement.NAME + "\\s+" + ATTRIBUTE_NAME + "\\s+"
			+ ATTRIBUTE_TYPE + "\\s+" + DEVAULT_VALUE + ">";

	private String string = null;

	/**
	 * Patter object compiled from ATTLIST_PATTERN
	 */
	public static Pattern attlistPattern = Pattern.compile(ATTLIST_PATTERN);

	/**
	 * Check wether attlist type is an ID type.
	 * 
	 * @return true or false
	 */
	public boolean isTypeID() {
		return ID.equals(getAttributeType());
	}

	/**
	 * Check wether attlist type is an IDREF type.
	 * 
	 * @return true or false
	 */
	public boolean isTypeIDREF() {
		return IDREF.equals(getAttributeType());
	}

	/**
	 * Check wether attlist type is an IDREFS type.
	 * 
	 * @return true or false
	 */
	public boolean isTypeIDREFS() {
		return IDREFS.equals(getAttributeType());
	}

	/**
	 * Check wether attlist type is an NMTOKEN type.
	 * 
	 * @return true or false
	 */
	public boolean isTypeNMTOKEN() {
		return NMTOKEN.equals(getAttributeType());
	}

	/**
	 * Check wether attlist type is an NMTOKENS type.
	 * 
	 * @return true or false
	 */
	public boolean isTypeNMTOKENS() {
		return NMTOKENS.equals(getAttributeType());
	}

	/**
	 * Check wether attlist type is an ENTITY type.
	 * 
	 * @return true or false
	 */
	public boolean isTypeENTITY() {
		return ENTITY.equals(getAttributeType());
	}

	/**
	 * Check wether attlist type is an ENTITIES type.
	 * 
	 * @return true or false
	 */
	public boolean isTypeENTITIES() {
		return ENTITIES.equals(getAttributeType());
	}

	/**
	 * Check wether attlist type is an IDREF type.
	 * 
	 * @return true or false
	 */
	public boolean isTypeNOTATION() {
		return NOTATION.equals(getAttributeType());
	}

	/**
	 * Check wether attlist type is an CDATA type.
	 * 
	 * @return true or false
	 */
	public boolean isTypeCDATA() {
		return CDATA.equals(getAttributeType());
	}

	/**
	 * Check wether attlist type is an enum
	 * 
	 * @return true or false
	 */
	public boolean isTypeENUM() {
		return getAttributeType().startsWith("(");
	}

	/**
	 * Create new instance of DtdAttLIst.
	 * 
	 * @param str
	 *            String part of dtd data that is a AttList data.
	 */
	public DtdAttList(String str) {
		Matcher match = attlistPattern.matcher(str);
		if (match.find()) {
			this.string = str;
		} else {
			throw new IllegalArgumentException(
					"Provided string is not a valid DTD Attribute List : "
							+ str);
		}
	}

	/**
	 * Get the attribute name of AttList.
	 * 
	 * @return Attribute name
	 */
	public String getAttributeName() {
		Matcher match = attlistPattern.matcher(string);
		match.find();
		return match.group(3);
	}

	/**
	 * Get the element name.
	 * 
	 * @return element name
	 */
	public String getElementName() {
		Matcher match = attlistPattern.matcher(string);
		match.find();
		return match.group(1);
	}

	/**
	 * Get the attribute type.
	 * 
	 * @return attribute type.
	 */
	public String getAttributeType() {
		Matcher match = attlistPattern.matcher(string);
		match.find();
		return match.group(5);
	}

	/**
	 * Get the attribute default value.
	 * 
	 * @return attribute's default value.
	 */
	public String getAttributeDefaultValue() {
		Matcher match = attlistPattern.matcher(string);
		match.find();
		return match.group(9);
	}
}
