package org.xanot.dtd;

import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Object model of "Element" inside a DTD file.
 * 
 * 
 * @author Ferdinand Neman (newm4n_at_gmail.com)
 * 
 */
public class DtdElement {

	/**
	 * Element name pattern : <code>(\w([\w\-]*\w)*)</code>
	 */
	public static String NAME = "(\\w([\\w\\-]*\\w)*)";

	/**
	 * Empty element pattern : <code>EMPTY </code>
	 */
	private static String EMPTY = "EMPTY";

	/**
	 * Any element pattern : <code>ANY</code>
	 */
	private static String ANY = "ANY";

	/**
	 * Element pattern : <code>ELEMENT</code>
	 */
	private static String ELEMENT = "ELEMENT";

	/**
	 * PCDATA pattern : <code>PCDATA</code>
	 */
	public static String PCDATA = "#PCDATA";

	/**
	 * Greedy mark : <code>[\?\*\+]?</code>
	 */
	private static String GREEDY_MARK = "[\\?\\*\\+]?";

	/**
	 * Child inside element pattern : NAME+GREEDY_MARK
	 */
	public static String CHILD = NAME + GREEDY_MARK;

	/**
	 * Elements options pattern :
	 * \(\s*("+PCDATA+"|"+CHILD+")(\s*\|\s*"+CHILD+")+\s*\) + GREEDY_MARK
	 */
	public static String OPTIONS = "\\(\\s*(" + PCDATA + "|" + CHILD
			+ ")(\\s*\\|\\s*" + CHILD + ")+\\s*\\)" + GREEDY_MARK;

	/**
	 * Collection of child in element pattern : \(\s*("+ PCDATA +
	 * "|"+CHILD+"|"+OPTIONS+")(\s*,\s*("+ PCDATA +
	 * "|"+CHILD+"|"+OPTIONS+"))*\s*\)" + GREEDY_MARK
	 */
	public static String CHILDS = "\\(\\s*(" + PCDATA + "|" + CHILD + "|"
			+ OPTIONS + ")(\\s*,\\s*(" + PCDATA + "|" + CHILD + "|" + OPTIONS
			+ "))*\\s*\\)" + GREEDY_MARK;

	/**
	 * Element's Category pattern : ("+ANY+"|"+EMPTY+"|"+CHILDS +"|"+OPTIONS+")
	 */
	public static String CATEGORY = "(" + ANY + "|" + EMPTY + "|" + CHILDS
			+ "|" + OPTIONS + ")";

	/**
	 * Element's main pattern : <!"+ELEMENT+"\s+"+NAME+"\s+"+CATEGORY+">
	 */
	public static String ELEMENT_PATTERN = "<!" + ELEMENT + "\\s+" + NAME
			+ "\\s+" + CATEGORY + ">";

	private static Pattern elementPattern = Pattern.compile(ELEMENT_PATTERN);

	private static Pattern namePattern = Pattern.compile(ELEMENT_PATTERN);

	private static Pattern categoryPattern = Pattern.compile(CATEGORY);

	private static Pattern chidsPattern = Pattern.compile(CHILDS);

	private static Pattern optionsPattern = Pattern.compile(OPTIONS);

	private static Pattern optionExtractPattern = Pattern.compile(PCDATA + "|"
			+ CHILD);

	private static Pattern childsExtractPattern = Pattern.compile(PCDATA + "|"
			+ CHILD + "|" + OPTIONS);

	private String string = null;

	/**
	 * Create new instance of DtdElement.
	 * 
	 * @param string
	 *            Part of DTD data that is representing the "ELEMENT" part.
	 */
	public DtdElement(String string) {
		Matcher match = elementPattern.matcher(string);
		if (match.find()) {
			this.string = string;
		} else {
			throw new IllegalArgumentException(
					"Provided string is not a valid DTD Element : " + string);
		}
	}

	/**
	 * Get the element's name.
	 * 
	 * @return Element name.
	 */
	public String getElementName() {
		Matcher match = namePattern.matcher(string);
		match.find();
		return match.group(1);
	}

	/**
	 * Get the element's category
	 * 
	 * @return Category.
	 */
	public String getCategory() {
		Matcher match = categoryPattern.matcher(string);
		match.find();
		return match.group();
	}

	/**
	 * Check wether the category is any.
	 * 
	 * @return True if the category is any. False if other wise.
	 */
	public boolean isCategoryAny() {
		return getCategory().equals(ANY);
	}

	/**
	 * Check wether the category is empty.
	 * 
	 * @return True if the category is empty. false if other wise.
	 */
	public boolean isCategoryEmpty() {
		return getCategory().equals(EMPTY);
	}

	/**
	 * Check wether the category is is sequenced.
	 * 
	 * @return true if the category is sequenced. false if otherwise.
	 */
	public boolean isCategorySequence() {
		Matcher match = chidsPattern.matcher(getCategory());
		return match.find();
	}

	/**
	 * check wether the category have options.
	 * 
	 * @return true if the category have options. false if otherwise.
	 */
	public boolean isCategoryOptions() {
		Pattern optionpattern = Pattern.compile("\\A" + OPTIONS + "\\z");
		Matcher match = optionpattern.matcher(getCategory());
		return match.find();
	}

	/**
	 * Extract category options from string to string array.
	 * 
	 * @param optionCategory
	 *            The category options
	 * @return String array.
	 */
	public String[] extractCategoryOptions(String optionCategory) {
		ArrayList<String> arr = new ArrayList<String>();
		Matcher match = optionsPattern.matcher(optionCategory);
		if (match.find()) {
			Matcher match2 = optionExtractPattern.matcher(match.group());
			while (match2.find()) {
				arr.add(match2.group());
			}
			return arr.toArray(new String[arr.size()]);
		} else {
			return new String[0];
		}
	}

	/**
	 * Extract category sequence from a string sequence.
	 * 
	 * @param sequence
	 *            Sequence
	 * @return Object array.
	 */
	public Object[] extractCategorySequence(String sequence) {
		ArrayList<Object> arr = new ArrayList<Object>();
		Matcher match = chidsPattern.matcher(sequence);
		if (match.find()) {
			Matcher match2 = childsExtractPattern.matcher(match.group());
			while (match2.find()) {
				String str = match2.group();
				if (str.startsWith("(")) {
					arr.add(extractCategoryOptions(match2.group()));
				} else {
					arr.add(match2.group());
				}
			}
			return arr.toArray(new Object[arr.size()]);
		} else {
			return new Object[0];
		}
	}

	public String[] extractWords(String sequence) {
		ArrayList<String> arr = new ArrayList<String>();
		Pattern namePattern = Pattern.compile(NAME);
		Matcher match = namePattern.matcher(sequence);
		while (match.find()) {
			arr.add(match.group());
		}
		return arr.toArray(new String[arr.size()]);
	}
}
