/*
 * Copyright 2006 The Xanot team
 */
package org.xanot.structure;

/**
 * Create instance rule. This rule define what class should be instantiated when
 * it encounters an xml element start.
 * 
 * @author Ferdinand Neman (newm4n _at_ gmail.com)
 */
public class CreateInstanceRule extends XanotRule {
	private Class baseClass = null;

	private boolean root = false;

	private boolean repeatable = false;

	/**
	 * Creates a new CreateInstanceRule object.
	 * 
	 * @param path
	 *            Path name, should be equals to the xml open tag name.
	 * @param baseClass
	 *            The class to instantiate
	 */
	public CreateInstanceRule(String path, Class baseClass) {
		super(path);
		setBaseClass(baseClass);
	}

	/**
	 * Get the class to instantiate
	 * 
	 * @return Returns the baseClass.
	 */
	public Class getBaseClass() {
		return baseClass;
	}

	/**
	 * Set the class to instantiate
	 * 
	 * @param baseClass
	 *            The baseClass to set.
	 */
	public void setBaseClass(Class baseClass) {
		this.baseClass = baseClass;
	}

	/**
	 * Create an object instance based on the base class
	 * 
	 * @return Instance
	 * @throws InstantiationException
	 *             Thrown when instantiating class.
	 * @throws IllegalAccessException
	 *             Thrown when the class is not legally accessed.
	 */
	public Object createInstance() throws InstantiationException,
			IllegalAccessException {
		return baseClass.newInstance();
	}

	/**
	 * Check wether the base class is the root class.
	 * 
	 * @return Returns the root.
	 */
	public boolean isRoot() {
		return root;
	}

	/**
	 * Determine wether the base class is the root class.
	 * 
	 * @param root
	 *            The root to set.
	 */
	public void setRoot(boolean root) {
		this.root = root;
	}

	/**
	 * Check wether the base class can be nested inside the same or child of
	 * this class
	 * 
	 * @return Returns the repeatable.
	 */
	public boolean isRepeatable() {
		return repeatable;
	}

	/**
	 * Determine wether the base class can be nested inside the same or child of
	 * this class
	 * 
	 * @param repeatable
	 *            The repeatable to set.
	 */
	public void setRepeatable(boolean repeatable) {
		this.repeatable = repeatable;
	}

	/**
	 * Get string representation of this object.
	 * 
	 * @return String representation.
	 */
	public String toString() {
		return "CreateInstanceRule : [path:" + getPath() + "],[baseClass:"
				+ baseClass.getName() + "],[root:" + root + "],[repeatable:"
				+ repeatable + "]";
	}
}
