/*
 * Copyright 2006 The Xanot team
 */
package org.xanot.util;

import java.util.ArrayList;
import java.util.Stack;
import java.util.StringTokenizer;

/**
 * A stack that support object to object mapping.
 * 
 * @author Ferdinand Neman (newm4n _at_ gmail.com)
 */
public class StackMap<SK, SV> {
	private Stack<MapElement<SK, SV>> stack = new Stack<MapElement<SK, SV>>();

	/**
	 * DOCUMENT ME!
	 * 
	 * @return DOCUMENT ME!
	 */
	public int size() {
		return stack.size();
	}

	/**
	 * <p>
	 * An additional method tocheck wether this stack is applicable for a
	 * specific path like data.
	 * </p>
	 * <p/>
	 * <p>
	 * For example :
	 * </p>
	 * <p/>
	 * 
	 * <pre><code>
	 *  Stack Key
	 *  +---+---+---+-----
	 *  | a | b | c |  &lt;--push  --&gt;pop
	 *  +---+---+---+-----
	 *  Is applicable to :
	 *     &quot;/a/b/c&quot; or &quot;../a/b/c&quot; or
	 *     &quot;../b/c&quot; or &quot;../c&quot;  or even just &quot;..&quot;
	 *  Not applicable to :
	 *     &quot;/1/b/c&quot; or &quot;/1/a/b/c&quot; or
	 *     &quot;../1/b/c&quot; or an empty string &quot;&quot;
	 * </pre><code>
	 *
	 * @param path Path string
	 * @return True if the path is applicable to the stack. False if otherwise.
	 */
	public boolean isApplicableForPath(String path) {
		if (path == null) {
			throw new NullPointerException();
		}

		if (path.trim().length() == 0) {
			return false;
		}

		ArrayList<String> strs = new ArrayList<String>();
		StringTokenizer stok = new StringTokenizer(path, "/");

		while (stok.hasMoreTokens()) {
			strs.add(stok.nextToken());
		}

		if (strs.get(0).equals("..")) {
			if (strs.size() > (stack.size() + 1)) {
				return false;
			} else if (strs.size() > stack.size()) {
				int stackIdx = stack.size() - 1;
				int pathIdx = strs.size() - 1;

				while ((stackIdx >= 0) && (pathIdx > 0)) {
					if (!strs.get(pathIdx).equals(getKey(stackIdx))) {
						return false;
					} else {
						stackIdx--;
						pathIdx--;
					}
				}

				return true;
			} else if (strs.size() <= stack.size()) {
				int stackIdx = stack.size() - 1;
				int pathIdx = strs.size() - 1;

				while ((stackIdx > 0) && (pathIdx >= 0)) {
					if (strs.get(pathIdx).equals("..")) {
						return true;
					} else if (!strs.get(pathIdx).equals(getKey(stackIdx))) {
						return false;
					} else {
						stackIdx--;
						pathIdx--;
					}
				}

				return true;
			} else {
				return false;
			}
		} else {
			if (strs.size() != stack.size()) {
				return false;
			} else {
				for (int i = 0; i < strs.size(); i++) {
					if (!strs.get(i).equals(getKey(i))) {
						return false;
					}
				}

				return true;
			}
		}
	}

	/**
	 * Add new map on top of the stack.
	 * 
	 * @param key
	 *            Key object
	 * @param value
	 *            Value object
	 */
	public void push(SK key, SV value) {
		MapElement<SK, SV> me = new MapElement<SK, SV>();
		me.setKey(key);
		me.setValue(value);
		stack.push(me);
	}

	/**
	 * Get and remove the map on top of the stack.
	 * 
	 * @return Mapped element.
	 */
	public MapElement<SK, SV> pop() {
		return stack.pop();
	}

	/**
	 * Get key path as string path.
	 * 
	 * @return Keypath through the stack.
	 */
	public String getStringKeyPath() {
		StringBuilder sb = new StringBuilder();

		for (SK key : getKeyPath()) {
			sb.append("/");
			sb.append(key.toString());
		}

		return sb.toString();
	}

	/**
	 * Get path of Key
	 * 
	 * @return return key path.
	 */
	public ArrayList<SK> getKeyPath() {
		ArrayList<SK> al = new ArrayList<SK>();

		for (int i = 0; i < stack.size(); i++) {
			al.add(stack.get(i).getKey());
		}

		return al;
	}

	/**
	 * Get path of Value
	 * 
	 * @return return value path.
	 */
	public ArrayList<SV> getValuePath() {
		ArrayList<SV> al = new ArrayList<SV>();

		for (int i = 0; i < stack.size(); i++) {
			al.add(stack.get(i).getValue());
		}

		return al;
	}

	/**
	 * Get the value of the map at specified index.
	 * 
	 * @param idx
	 *            Index.
	 * @return The value.
	 */
	public SV getValue(int idx) {
		return stack.get(idx).getValue();
	}

	/**
	 * Get the key of the map at specified index.
	 * 
	 * @param idx
	 *            Index
	 * @return The key
	 */
	public SK getKey(int idx) {
		return stack.get(idx).getKey();
	}

	/**
	 * Get the value of map right below the top-most stack.
	 * 
	 * @return The value.
	 */
	public SV peekParentValue() {
		if (stack.size() > 1) {
			return stack.get(stack.size() - 2).getValue();
		} else {
			System.out.println("Parent value null");

			return null;
		}
	}

	/**
	 * Get the key of map right below the top-most stack.
	 * 
	 * @return The key.
	 */
	public SK peekParentKey() {
		if (stack.size() > 1) {
			return stack.get(stack.size() - 2).getKey();
		} else {
			return null;
		}
	}

	/**
	 * Check wether a key is already inside the stack map.
	 * 
	 * @param key
	 *            Key to check
	 * @return True if the key already exist. False if otherwise.
	 */
	public boolean containsKey(SK key) {
		for (MapElement<SK, SV> me : stack) {
			if (me.getKey().equals(key)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Check wether a value is already inside the stack map.
	 * 
	 * @param value
	 *            Value to check
	 * @return True if the value already exist. False if otherwise.
	 */
	public boolean containsValue(SV value) {
		for (MapElement<SK, SV> me : stack) {
			if (me.getValue().equals(value)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Inner class.
	 * 
	 * @author ferdi
	 * 
	 * @param <K>
	 *            Key template
	 * @param <V>
	 *            Value template
	 */
	class MapElement<K, V> {
		private K key;

		private V value;

		public MapElement() {
		}

		public V getValue() {
			return value;
		}

		public void setValue(V data) {
			value = data;
		}

		public K getKey() {
			return key;
		}

		public void setKey(K data) {
			key = data;
		}
	}
}
